@@ -13,6 +13,7 @@
 #include <QRadioButton>
 #include <QPushButton>
 #include <QSpinBox>
+#include <QLineEdit>
 #include <QTabWidget>
 #include <QVBoxLayout>
 #include <QGridLayout>
@@ -49,15 +50,18 @@ QString font_string(std::string str) {
 class PreferencesPageScroll : public QScrollArea {
 	protected:
 		QWidget *child_widget;
-		bool use_child_hint;
+		bool use_child_hint_h, use_child_hint_w;
 	public:
-		PreferencesPageScroll(QWidget *parent, QWidget *child, bool b) : QScrollArea(parent), child_widget(child), use_child_hint(b) {
+		PreferencesPageScroll(QWidget *parent, QWidget *child, bool bh, bool bw) : QScrollArea(parent), child_widget(child), use_child_hint_h(bh), use_child_hint_w(bw) {
 			setWidget(child);
 		}
 		QSize sizeHint() const {
-			if(use_child_hint) {
+			if(use_child_hint_w || use_child_hint_h) {
 				QSize s = child_widget->sizeHint();
-				s.setHeight(s.height() + 1);
+				if(use_child_hint_w) s.setWidth(s.width() + 2);
+				else s.setWidth(QScrollArea::sizeHint().width());
+				if(use_child_hint_h) s.setHeight(s.height() + 2);
+				else s.setHeight(QScrollArea::sizeHint().height());
 				return s;
 			}
 			return QScrollArea::sizeHint();
@@ -117,6 +121,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(langChanged(int)));
 	l->addWidget(combo, r, 1); r++; langCombo = combo;
 	BOX(tr("Allow multiple instances"), settings->allow_multiple_instances > 0, multipleInstancesToggled(bool));
+	BOX(tr("Notify when a new version is available"), settings->check_version, checkVersionToggled(bool));
 	BOX(tr("Clear history on exit"), settings->clear_history_on_exit, clearHistoryToggled(bool));
 	l->addWidget(new QLabel(tr("Max history lines saved:"), this), r, 0); QSpinBox *spin = new QSpinBox(this); spin->setRange(0, 100000); spin->setValue(settings->max_history_lines); connect(spin, SIGNAL(valueChanged(int)), this, SLOT(maxHistoryLinesChanged(int))); l->addWidget(spin, r, 1); r++;
 	BOX(tr("Close application with Escape key"), settings->close_with_esc, closeWithEscToggled(bool));
@@ -132,7 +137,7 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 		box2->setChecked(settings->preserve_history_height > 0);
 	}
 	box2->setToolTip(tr("Do not change the height of history list when keypad or number bases are show or hidden."));
-	BOX(tr("Place expression field below history (experimental)"), settings->expression_pos == 1, expressionPositionToggled(bool));
+	BOX(tr("Place expression field below history"), settings->expression_pos == 1, expressionPositionToggled(bool));
 	BOX(tr("Show status bar"), settings->show_statusbar || settings->status_in_statusbar, showStatusBarToggled(bool));
 	l->addWidget(new QLabel(tr("Window title:"), this), r, 0);
 	combo = new QComboBox(this);
@@ -209,12 +214,15 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	calculateDelayWidget->setEnabled(settings->status_in_history);
 	connect(calculateDelayWidget, SIGNAL(valueChanged(int)), this, SLOT(calculateDelayChanged(int)));
 	l->addWidget(calculateDelayWidget, r, 1); r++;
-	BOX(tr("Automatically calculate selection"), settings->autocalc_selection, autocalcSelectionToggled(bool));
+	BOX(tr("Adaptive delay"), settings->adaptive_autocalc_delay, adaptiveDelayToggled(bool)); box->setEnabled(settings->status_in_history); adaptiveDelayBox = box;
+	BOX(tr("Automatically calculate selection"), settings->autocalc_selection, autocalcSelectionToggled(bool)); autocalcSelectionBox = box;
 	l->addWidget(new QLabel(tr("Expression in history:"), this), r, 0);
 	combo = new QComboBox(this);
-	combo->addItem(tr("Parsed"), 0);
-	combo->addItem(tr("Entered"), 1);
-	combo->addItem(tr("Entered + parsed"), 2);
+	combo->addItem(tr("Parsed"), HISTORY_EXPRESSION_TYPE_PARSED);
+	combo->addItem(tr("Parsed (compact)"), HISTORY_EXPRESSION_TYPE_PARSED_COMPACT);
+	combo->addItem(tr("Entered"), HISTORY_EXPRESSION_TYPE_ENTERED);
+	combo->addItem(tr("Entered + parsed"), HISTORY_EXPRESSION_TYPE_ENTERED_AND_PARSED);
+	combo->addItem(tr("Entered + parsed (compact)"), HISTORY_EXPRESSION_TYPE_ENTERED_AND_PARSED_COMPACT);
 	combo->setCurrentIndex(combo->findData(settings->history_expression_type));
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(historyExpressionChanged(int)));
 	l->addWidget(combo, r, 1); r++;
@@ -318,9 +326,15 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	combo->addItem(tr("None"), DIGIT_GROUPING_NONE);
 	combo->addItem(tr("Standard"), DIGIT_GROUPING_STANDARD);
 	combo->addItem(tr("Local"), DIGIT_GROUPING_LOCALE);
-	combo->setCurrentIndex(combo->findData(settings->printops.digit_grouping));
+	combo->addItem(tr("Custom"), DIGIT_GROUPING_LOCALE + 1);
+	combo->setCurrentIndex(combo->findData(settings->custom_digit_grouping ? DIGIT_GROUPING_LOCALE + 1 : settings->printops.digit_grouping));
 	connect(combo, SIGNAL(currentIndexChanged(int)), this, SLOT(groupingChanged(int)));
 	l->addWidget(combo, r, 1); r++;
+	QHBoxLayout *cgl = new QHBoxLayout();
+	spin = new QSpinBox(this); spin->setRange(settings->custom_digit_grouping ? 1 : 0, 70); spin->setValue(settings->max_history_lines); connect(spin, SIGNAL(valueChanged(int)), this, SLOT(cgfChanged(int))); cgl->addWidget(spin); cgfSpin = spin;
+	cgsEdit = new QLineEdit(this); connect(cgsEdit, SIGNAL(textEdited(const QString&)), this, SLOT(cgsChanged(const QString&))); cgl->addWidget(cgsEdit);
+	updateCustomDigitGrouping();
+	l->addLayout(cgl, r, 1); r++;
 	BOX(tr("Automatically group digits in input"), settings->automatic_digit_grouping, automaticDigitGroupingToggled(bool));
 	l->addWidget(new QLabel(tr("Interval display:"), this), r, 0);
 	combo = new QComboBox(this);
@@ -468,13 +482,14 @@ PreferencesDialog::PreferencesDialog(QWidget *parent) : QDialog(parent) {
 	if(settings->always_on_top) setWindowFlags(windowFlags() | Qt::WindowStaysOnTopHint);
 	QSize size = get_screen_geometry(this).size();
 	QSize sh = w1->sizeHint().expandedTo(w2->sizeHint().expandedTo(w3->sizeHint().expandedTo(w4->sizeHint().expandedTo(w5->sizeHint()))));
-	if(size.width() >= sh.width() + 100) tabs->setUsesScrollButtons(false);
-	bool use_child_hint = size.height() >= sh.height() + 100;
-	QScrollArea *s1 = new PreferencesPageScroll(this, w1, use_child_hint);
-	QScrollArea *s2 = new PreferencesPageScroll(this, w2, use_child_hint);
-	QScrollArea *s3 = new PreferencesPageScroll(this, w3, use_child_hint);
-	QScrollArea *s4 = new PreferencesPageScroll(this, w4, use_child_hint);
-	QScrollArea *s5 = new PreferencesPageScroll(this, w5, use_child_hint);
+	bool use_child_hint_h = size.height() >= sh.height() + 100;
+	bool use_child_hint_w = size.width() >= sh.width() + 100;
+	if(use_child_hint_w) tabs->setUsesScrollButtons(false);
+	QScrollArea *s1 = new PreferencesPageScroll(this, w1, use_child_hint_h, use_child_hint_w);
+	QScrollArea *s2 = new PreferencesPageScroll(this, w2, use_child_hint_h, use_child_hint_w);
+	QScrollArea *s3 = new PreferencesPageScroll(this, w3, use_child_hint_h, use_child_hint_w);
+	QScrollArea *s4 = new PreferencesPageScroll(this, w4, use_child_hint_h, use_child_hint_w);
+	QScrollArea *s5 = new PreferencesPageScroll(this, w5, use_child_hint_h, use_child_hint_w);
 	tabs->addTab(s1, tr("Look && Feel"));
 	tabs->addTab(s5, tr("Fonts"));
 	tabs->addTab(s2, tr("Numbers && Operators"));
@@ -519,6 +534,9 @@ void PreferencesDialog::langChanged(int i) {
 void PreferencesDialog::multipleInstancesToggled(bool b) {
 	settings->allow_multiple_instances = b;
 }
+void PreferencesDialog::checkVersionToggled(bool b) {
+	settings->check_version = b;
+}
 void PreferencesDialog::clearHistoryToggled(bool b) {
 	settings->clear_history_on_exit = b;
 }
@@ -561,6 +579,10 @@ void PreferencesDialog::statusModeChanged(int i) {
 	settings->status_in_statusbar = (i == 3);
 	statusDelayWidget->setEnabled(settings->display_expression_status);
 	calculateDelayWidget->setEnabled(settings->status_in_history);
+	adaptiveDelayBox->setEnabled(settings->status_in_history);
+	adaptiveDelayBox->blockSignals(true);
+	adaptiveDelayBox->setChecked(settings->status_in_history && settings->adaptive_autocalc_delay);
+	adaptiveDelayBox->blockSignals(false);
 	emit statusModeChanged();
 }
 void PreferencesDialog::statusDelayChanged(int v) {
@@ -569,6 +591,9 @@ void PreferencesDialog::statusDelayChanged(int v) {
 void PreferencesDialog::calculateDelayChanged(int v) {
 	settings->auto_calculate_delay = v;
 }
+void PreferencesDialog::adaptiveDelayToggled(bool b) {
+	settings->adaptive_autocalc_delay = b;
+}
 void PreferencesDialog::autocalcSelectionToggled(bool b) {
 	settings->autocalc_selection = b;
 }
@@ -830,9 +855,46 @@ void PreferencesDialog::updateVariableUnits() {
 	variableUnitsBox->setChecked(CALCULATOR->variableUnitsEnabled());
 	variableUnitsBox->blockSignals(false);
 }
+void PreferencesDialog::updateCustomDigitGrouping() {
+	cgfSpin->blockSignals(true);
+	cgsEdit->blockSignals(true);
+	if(settings->custom_digit_grouping) {
+		cgfSpin->setMinimum(1);
+		cgfSpin->setValue(settings->custom_digit_group_format.empty() ? 3 : settings->custom_digit_group_format[0] - '0');
+		cgsEdit->setText(QString::fromStdString(settings->custom_digit_group_separator));
+	} else {
+		cgfSpin->setMinimum(0);
+		cgfSpin->setValue(settings->printops.digit_grouping == DIGIT_GROUPING_NONE ? 0 : (settings->printops.digit_grouping == DIGIT_GROUPING_STANDARD || CALCULATOR->local_digit_group_format.empty() || CALCULATOR->local_digit_group_format[0] == CHAR_MAX ? 3 : (int) CALCULATOR->local_digit_group_format[0]));
+		cgsEdit->setText(settings->printops.digit_grouping == DIGIT_GROUPING_NONE ? QString() : (settings->printops.digit_grouping == DIGIT_GROUPING_STANDARD ? THIN_SPACE : QString::fromStdString(CALCULATOR->local_digit_group_separator)));
+	}
+	cgfSpin->blockSignals(false);
+	cgsEdit->blockSignals(false);
+	cgfSpin->setEnabled(settings->custom_digit_grouping);
+	cgsEdit->setEnabled(settings->custom_digit_grouping);
+}
 void PreferencesDialog::groupingChanged(int i) {
-	settings->printops.digit_grouping = (DigitGrouping) qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
-	if(settings->printops.digit_grouping == DIGIT_GROUPING_LOCALE && (!settings->evalops.parse_options.comma_as_separator || CALCULATOR->getDecimalPoint() == COMMA) && CALCULATOR->local_digit_group_separator == COMMA) {
+	int v = qobject_cast<QComboBox*>(sender())->itemData(i).toInt();
+	if(settings->custom_digit_grouping) {
+		CALCULATOR->local_digit_group_separator = settings->saved_local_dgs;
+		CALCULATOR->local_digit_group_format = settings->saved_local_dgf;
+	} else if(v == DIGIT_GROUPING_LOCALE + 1) {
+		settings->saved_local_dgs = CALCULATOR->local_digit_group_separator;
+		settings->saved_local_dgf = CALCULATOR->local_digit_group_format;
+	}
+	if(v == DIGIT_GROUPING_LOCALE + 1) {
+		settings->printops.digit_grouping = DIGIT_GROUPING_LOCALE;
+		CALCULATOR->local_digit_group_format = "";
+		for(size_t i = 0; i < settings->custom_digit_group_format.size(); i++) {
+			CALCULATOR->local_digit_group_format += settings->custom_digit_group_format[i] - '0';
+		}
+		CALCULATOR->local_digit_group_separator = settings->custom_digit_group_separator;
+		settings->custom_digit_grouping = true;
+	} else {
+		settings->printops.digit_grouping = (DigitGrouping) v;
+		settings->custom_digit_grouping = false;
+	}
+	updateCustomDigitGrouping();
+	if(v == DIGIT_GROUPING_LOCALE && (!settings->evalops.parse_options.comma_as_separator || CALCULATOR->getDecimalPoint() == COMMA) && CALCULATOR->local_digit_group_separator == COMMA) {
 		if(CALCULATOR->getDecimalPoint() == COMMA) {
 			settings->evalops.parse_options.comma_as_separator = true;
 			settings->evalops.parse_options.dot_as_separator = false;
@@ -846,7 +908,7 @@ void PreferencesDialog::groupingChanged(int i) {
 		} else {
 			ignoreCommaBox->toggle();
 		}
-	} else if(settings->printops.digit_grouping == DIGIT_GROUPING_LOCALE && (!settings->evalops.parse_options.dot_as_separator || CALCULATOR->getDecimalPoint() == DOT) && CALCULATOR->local_digit_group_separator == DOT) {
+	} else if(v == DIGIT_GROUPING_LOCALE && (!settings->evalops.parse_options.dot_as_separator || CALCULATOR->getDecimalPoint() == DOT) && CALCULATOR->local_digit_group_separator == DOT) {
 		if(CALCULATOR->getDecimalPoint() == DOT) {
 			settings->evalops.parse_options.dot_as_separator = true;
 			settings->evalops.parse_options.comma_as_separator = false;
@@ -864,6 +926,23 @@ void PreferencesDialog::groupingChanged(int i) {
 		emit resultFormatUpdated();
 	}
 }
+void PreferencesDialog::cgfChanged(int i) {
+	if(!settings->custom_digit_grouping) return;
+	settings->custom_digit_group_format = '0' + i;
+	CALCULATOR->local_digit_group_format = "";
+	for(size_t i = 0; i < settings->custom_digit_group_format.size(); i++) {
+		CALCULATOR->local_digit_group_format += settings->custom_digit_group_format[i] - '0';
+	}
+	settings->custom_digit_group_changed = true;
+	emit resultFormatUpdated();
+}
+void PreferencesDialog::cgsChanged(const QString &str) {
+	if(!settings->custom_digit_grouping) return;
+	settings->custom_digit_group_separator = str.toStdString();
+	CALCULATOR->local_digit_group_separator = settings->custom_digit_group_separator;
+	settings->custom_digit_group_changed = true;
+	emit resultFormatUpdated();
+}
 void PreferencesDialog::intervalDisplayChanged(int i) {
 	if(i == 0) {
 		settings->printops.interval_display = INTERVAL_DISPLAY_SIGNIFICANT_DIGITS;
@@ -1062,6 +1141,8 @@ void PreferencesDialog::updateExpressionStatus() {
 	statusDelayWidget->blockSignals(true);
 	calculateDelayWidget->blockSignals(true);
 	statusCombo->blockSignals(true);
+	autocalcSelectionBox->blockSignals(true);
+	adaptiveDelayBox->blockSignals(true);
 	if(!settings->display_expression_status) statusCombo->setCurrentIndex(0);
 	else if(settings->status_in_history) statusCombo->setCurrentIndex(1);
 	else if(settings->status_in_statusbar) statusCombo->setCurrentIndex(3);
@@ -1070,9 +1151,14 @@ void PreferencesDialog::updateExpressionStatus() {
 	statusDelayWidget->setEnabled(settings->display_expression_status);
 	calculateDelayWidget->setValue(settings->auto_calculate_delay);
 	calculateDelayWidget->setEnabled(settings->status_in_history);
+	adaptiveDelayBox->setEnabled(settings->status_in_history);
+	adaptiveDelayBox->setChecked(settings->status_in_history && settings->adaptive_autocalc_delay);
+	autocalcSelectionBox->setChecked(settings->autocalc_selection);
 	statusDelayWidget->blockSignals(false);
 	calculateDelayWidget->blockSignals(false);
 	statusCombo->blockSignals(false);
+	autocalcSelectionBox->blockSignals(false);
+	adaptiveDelayBox->blockSignals(false);
 }
 void PreferencesDialog::updateTemperatureCalculation() {
 	tcCombo->blockSignals(true);
