@@ -20,6 +20,7 @@ class QAbstractButton;
 class QCheckBox;
 class QSpinBox;
 class QComboBox;
+class QLineEdit;
 
 class PreferencesDialog : public QDialog {
 
@@ -27,10 +28,11 @@ class PreferencesDialog : public QDialog {
 
 	protected:
 
-		QAbstractButton *decimalCommaBox, *ignoreCommaBox, *ignoreDotBox, *ignoreLocaleBox, *variableUnitsBox, *conciseUncertaintyInputBox, *rpnKeysBox;
+		QAbstractButton *decimalCommaBox, *ignoreCommaBox, *ignoreDotBox, *ignoreLocaleBox, *variableUnitsBox, *conciseUncertaintyInputBox, *rpnKeysBox, *adaptiveDelayBox, *autocalcSelectionBox;
 		QCheckBox *preserveHeightBox;
-		QSpinBox *exratesSpin, *statusDelayWidget, *calculateDelayWidget;
+		QSpinBox *exratesSpin, *statusDelayWidget, *calculateDelayWidget, *cgfSpin;
 		QComboBox *styleCombo, *parseCombo, *tcCombo, *langCombo, *complexFormCombo, *intervalDisplayCombo, *intervalCalculationCombo, *statusCombo;
+		QLineEdit *cgsEdit;
 
 		void closeEvent(QCloseEvent*) override;
 
@@ -44,6 +46,7 @@ class PreferencesDialog : public QDialog {
 		void statusDelayChanged(int);
 		void autocalcSelectionToggled(bool);
 		void calculateDelayChanged(int);
+		void adaptiveDelayToggled(bool);
 		void binTwosToggled(bool);
 		void hexTwosToggled(bool);
 		void binTwosInputToggled(bool);
@@ -82,7 +85,10 @@ class PreferencesDialog : public QDialog {
 		void allPrefixesToggled(bool);
 		void denominatorPrefixToggled(bool);
 		void variableUnitsToggled(bool);
+		void updateCustomDigitGrouping();
 		void groupingChanged(int);
+		void cgfChanged(int);
+		void cgsChanged(const QString&);
 		void intervalDisplayChanged(int);
 		void conciseUncertaintyInputToggled(bool);
 		void limitImplicitToggled(bool);
@@ -105,6 +111,7 @@ class PreferencesDialog : public QDialog {
 		void replaceExpressionChanged(int);
 		void autocopyResultToggled(bool);
 		void multipleInstancesToggled(bool);
+		void checkVersionToggled(bool);
 		void clearHistoryToggled(bool);
 		void maxHistoryLinesChanged(int);
 		void historyExpressionChanged(int);
